<?php

namespace Contabilidade\Logic\LancamentoContabil;

use \Geral\Entities\Lanccont;
use \Setforms2\Shared\Registry;
use \Geral\Util\DateUtil;
use Geral\Util\TratamentoDeMensagens;
use \Geral\Util\Ambiente;
use Setforms2\Runtime\Framework;

class LancamentoContabil
{
    const QTDE = 'QTDE';
    const CONTA_REDUZIDA = 'CDCONTREDU';
    const CONTA_CONTABIL = 'CDCONTCTBL';
    const NOME_CONTA_CONTABIL = 'NMCONTCTBL';
    const CREDITO = 'VRCREDITO';
    const DEBITO = 'VRDEBITO';
    const HISTORICO = 'CDHISTPADR';
    const COMERCIAL = 'CDCONTCOME';
    const ORGANIZACAO = 'NRORG';
    const IDTIPOLANC = 'IDTIPOLANC';
    const NRLANCCONT = 'NRLANCCONT';
    const DTLANCCONT = 'DTLANCCONT';
    const CDDOCUCONT = 'CDDOCUCONT';
    const IDLANCCOME = 'IDLANCCOME';
    const EMPRESA = 'CDEMPRESA';
    const SEQUENCIAL = 'NRSEQLANC';
    const DESCRICAO_HIST = 'DSHISTLANC';
    const WINDOW_PRINCIPAL = 'win_pbi00229_000';

    public static function insereLancamento($campos, $data, $documento){
        try {
            foreach ($campos as $campo) {
                $lanccont = new Lanccont();
                $lanccont->setCdempresa($campo['CDEMPRESA']);
                $lanccont->setNrlanccont($campo['NRLANCCONT']);
                $lanccont->setNrseqlanc($campo['NRSEQLANC']);
                $lanccont->setCdcontctbl($campo['CDCONTCTBL']);
                $lanccont->setCdhistpadr($campo['CDHISTPADR']);
                $lanccont->setDshistlanc($campo['DSHISTLANC']);
                $lanccont->setDtlanccont($campo['DTLANCCONT']);
                $lanccont->setCddocucont($documento);
                $lanccont->setIdtipolanc($campo['IDTIPOLANC']);
                $lanccont->setDshistlanc($campo['DSHISTLANC']);
                $lanccont->setVrlancament($campo['VRLANCAMENT']);
                $lanccont->setIdlanccome($campo['IDLANCCOME']);
                $lanccont->setCdcontcome($campo['CDCONTCOME']);
                $lanccont->setIdlancinte('M');
                $lanccont->setNrorg($campo['NRORG']);
                $lanccont->setDtinclusao($data);
                $lanccont->setNrorginclusao($campo['NRORG']);
                $lanccont->setCdoperinclusao($campo['CDOPERADOR']);
                $lanccont->setIdativo('S');
                $lanccont->persist();
                Registry::getEntityManager()->flush();
            }
        }catch(\Exception $e){
            throw $e;
        }
    }

    /**
     * Exclui o lançamento completo direto do banco.
     * @throws \Exception
     */
    static public function excluirLancamento($nrorg, $nrlanccont, $empresa){
        try {
            $lancamentos = Lanccont::findByNrlanccontAndNrorgAndCdempresa($nrlanccont, $nrorg, $empresa);
            foreach($lancamentos as $lancamento)
                $lancamento->delete();
            Registry::getEntityManager()->flush();
        }catch (\Exception $e)
        {
            throw $e;
        }
    }

    public static function partidaAtiva(){
        $tabId = Framework::getTabId();
        $registro = tk_get_field_value('QTDE');
        tk_set_env_var('PARTIDA_ATIVA', $tabId);
        tk_set_env_var('REGISTRO_ATIVO', $registro);
    }

    public static function atualizaHistorico(){
        $tab_ativa = substr(tk_get_env_var('PARTIDA_ATIVA'), 17);
        tk_set_field_value('CDHISTPADR', tk_get_field_value('CDHISTPADR'), $tab_ativa);
//        tk_set_field_value('DSHISTLANC', tk_get_field_value('DSHISTPADR'), $tab_ativa);
        tk_set_field_value('DSHISTLANC', tk_get_field_value('DSHISTPADRAO'), $tab_ativa);
    }

    public static function formataData($data){
        if(strlen($data) == 19)
            $data = DateUtil::truncateString($data);
        $data = DateUtil::getDatadeString($data);
        $data = DateUtil::truncateData($data);

        return $data;
    }

    /**
     * Atualiza a cada alteração das partidas de crédito
     * e débito os valores da tab_panel documentos.
     */
    public static function atualizaDocumento(){
        $data_cred = tk_get_env_var('DATA_CREDITO');
        $mapper_cred = tk_get_env_var('MAPPER_CREDITO');
        $mapper_cred = $mapper_cred->getDatasource()->getData();
        $data_deb = tk_get_env_var('DATA_DEBITO');
        $mapper_deb = tk_get_env_var('MAPPER_DEBITO');
        $mapper_deb = $mapper_deb->getDatasource()->getData();
        self::atualizaData($mapper_cred, $data_cred, 'PARTIDAS_CREDITO');
        self::atualizaData($mapper_deb, $data_deb, 'PARTIDAS_DEBITO');
        $valor_credito = 0;
        $valor_debito = 0;
        $partidas_cred = tk_get_env_var('PARTIDAS_CREDITO');
        $partidas_deb = tk_get_env_var('PARTIDAS_DEBITO');
        foreach($partidas_cred as $data) {
            $valor_credito += $data['VRCREDITO'];
        }
        foreach($partidas_deb as $data) {
            $valor_debito += $data['VRDEBITO'];
        }
        $diferenca = floatval(abs($valor_credito - $valor_debito));
        tk_set_field_value('VRCREDITO', $valor_credito, 'tab_pbi00229_000_005');
        tk_set_field_value('VRDEBITO', $valor_debito, 'tab_pbi00229_000_005');
        tk_set_field_value('DIFCREDDEB', $diferenca, 'tab_pbi00229_000_005');
    }

    /**
     * @param $itensAnt
     * @param $itensNov
     * @param $tab_ativa
     * Atualiza dados do panel de acordo com partida excluída
     */
    public static function atualizaData($itensAnt, $itensNov, $tipo_partida)
    {
        if (!empty($itensNov)) {
            for ($p = 0; count($itensNov) > $p; $p++) {
                for ($i = 0; count($itensAnt) > $i; $i++) {
                    if (count($itensNov) > 0 && $itensNov[$p][self::QTDE] == $itensAnt[$i][self::QTDE]) {
                        array_splice($itensAnt, $i, 1, array($itensNov[$p]));
                        array_splice($itensNov, $p, 1);
                    }
                }
            }
            if (!empty($itensNov) && count($itensNov[0]) > 0)
                array_splice($itensAnt, $i, 1, $itensNov);
        }
        tk_set_env_var($tipo_partida, $itensAnt);
    }

    public static function updateNomeDocumento($lancamento, $documento, $organizacao)
    {
        try {
            $status = false;
            $connection = \Setforms2\Shared\Registry::getEntityManager();
            $connection->beginTransaction();

            /** @var Lanccont $item */
            $lanccont = Lanccont::findByNrlanccontAndNrorg($lancamento, $organizacao);
            if (!empty($lanccont)) {
                foreach ($lanccont as $item) {
                    $item->setCddocucont($documento);
                    $item->persist();
                }
                $status = true;
            }
            $connection->commit();
            $connection->flush();
            return $status;
        } catch (\Exception $e) {
            $connection->rollback();
            TratamentoDeMensagens::exibirMensagens($e, true);
        }
    }
}
